// XWindow.h:
// -------------------------------------------------------------------
// This file is a part of "XTL" available at:
// http://www.codecommunity.com
//
// XTL is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
// 
// XTL is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with XTL. If not, see <http://www.gnu.org/licenses/>.
//
// coded by: skulk@codecommunity.com
// modified: 06-12-22 (skulk)
//////////////////////////////////////////////////////////////////////
#ifdef X_WINDOWS

#pragma once

#include <windows.h>

// For API based property pages...
#define _WTL_NEW_PAGE_NOTIFY_HANDLERS

// For "CString"...
#define _WTL_USE_CSTRING

#include <atlbase.h>

#include <atlapp.h>
#include <atlcrack.h>
#include <atlctrls.h>
#include <atlctrlw.h>
#include <atlctrlx.h>
#include <atlddx.h>
#include <atldlgs.h>
#include <atlframe.h>
#include <atlgdi.h>
#include <atlmisc.h>
#include <atlprint.h>
#include <atlres.h>

#ifdef _WIN32_WCE
	#include <atlresce.h>
#endif // _WIN32_WCE

#include <atlscrl.h>
#include <atlsplit.h>

#if (_WIN32_WINNT >= 0x0501)
	#include <atltheme.h>
#endif // (_WIN32_WINNT >= 0x0501)

#include <atluser.h>
#include <atlwinx.h>

#include <commctrl.h>
#include <shlwapi.h>

#pragma comment(lib, "comctl32")
#pragma comment(lib, "shlwapi")

#include "XBase.h"
#include "XLanguage.h"
#include "XWindowDefinitions.h"



// #######################################################################################



// ***************************************************************************************
// Resource loading macros...
// ***************************************************************************************
#define _XApplication_LoadResource(__Name, __Type) \
	inline __Type Load##__Name(xcchart* pszName) \
	{ \
		_XAssert(m_hModule || m_ModuleResource.IsValid()); \
		/* Try to load from resource dll... */ \
		__Type hType = NULL; \
		if (m_ModuleResource.IsValid()) \
		{ \
			hType = ::Load##__Name(m_ModuleResource.GetSafeType(), pszName); \
		} \
		/* Try to load from application resources... */ \
		if (!hType) \
		{ \
			hType = ::Load##__Name(m_hModule, pszName); \
		} \
		/* Try to load from predefined (system) resources... */ \
		if (!hType) \
		{ \
			hType = ::Load##__Name(NULL, pszName); \
		} \
		_XAssert(hType); \
		return hType; \
	} \
	inline __Type Load##__Name(xcuint32 uiID) \
	{ \
		return Load##__Name(MAKEINTRESOURCE(uiID)); \
	}	



// #######################################################################################



// ***************************************************************************************
// XApplication: Object handling resources...
// ***************************************************************************************
class XApplication : public CAppModule
{
// Base class...
typedef CAppModule _Base;

public:
// Construction...
	inline XApplication()
	{
		m_hModule = NULL;
	}

public:
// Operations...
	// Init application...
	inline xresult Init(xcchart* pszLanguage = NULL, xcchart* pszOEM = NULL, xvoid* pInstance = NULL)
	{
		// Shutdown first...
		if (IsInitialized())
		{
			Shutdown();
		}

		// Get module handle...
		m_hModule = pInstance ? ((HMODULE) pInstance) : ::GetModuleHandle(NULL);

		_XAssert(m_hModule);
		if (!m_hModule)
			return xePointer;

		_Base::Init(NULL, m_hModule, NULL);

		XPtrT<xchart> psz;
		xresult xr = _XPathRemoveExtension(XModuleInformation::GetFileName(m_hModule), &psz);
		if (_XFailed(xr))
			return xr;

		XString sFileName(psz);

		// User language...
		xcchart* pszLanguageUser = _XLanguageGetISOCodeByLangID(::GetUserDefaultLangID());

		// System language...
		xcchart* pszLanguageSystem = _XLanguageGetISOCodeByLangID(::GetSystemDefaultLangID());

		// Check if we can get a module for the passed language, if no language is passed or no module is available
		//  we try to get user and system default language modules...
		XPtrT<xchart> pszModule;
		if (_XFailed(xr = GetResourceFileName(sFileName, pszOEM, pszLanguage, &pszModule)) ||
			_XFailed(xr = m_ModuleResource.Load(pszModule)))
		{
			XPtrT<xchart> pszModule;
			if (_XFailed(xr = GetResourceFileName(sFileName, pszOEM, pszLanguageUser, &pszModule)) ||
				_XFailed(xr = m_ModuleResource.Load(pszModule)))
			{
				XPtrT<xchart> pszModule;
				if (_XSucceeded(xr = GetResourceFileName(sFileName, pszOEM, pszLanguageSystem, &pszModule)))
				{
					xr = m_ModuleResource.Load(pszModule);
				}
			}
		}
		
		return xsOK;
	}

	// Load common controls...
	inline xresult LoadCommonConrols()
	{
		INITCOMMONCONTROLSEX icc = 
		{
			sizeof(icc),
			ICC_ANIMATE_CLASS |
			ICC_BAR_CLASSES |
			ICC_COOL_CLASSES |
			ICC_DATE_CLASSES |
			ICC_HOTKEY_CLASS |
			ICC_INTERNET_CLASSES |
			ICC_LISTVIEW_CLASSES |
			ICC_NATIVEFNTCTL_CLASS |
			ICC_PAGESCROLLER_CLASS |
			ICC_PROGRESS_CLASS |
			ICC_TAB_CLASSES |
			ICC_TREEVIEW_CLASSES |
			ICC_UPDOWN_CLASS |
			ICC_USEREX_CLASSES |
			ICC_WIN95_CLASSES
		};

		return ::InitCommonControlsEx(&icc) ? xsOK : xeFail;
	}

	// Shutdown application...
	inline xresult Shutdown()
	{
		if (m_ModuleResource.IsValid())
		{
			m_ModuleResource.Free();
		}

		_Base::Term();

		m_hModule = NULL;

		return xsOK;
	}

	// Check if the application is initialized...
	inline xbool IsInitialized() const
	{
		return m_hModule || m_ModuleResource.IsValid();
	}

	// Get resource handle for a specific resource...
	inline HMODULE GetResourceHandle(xcuint32 uiID, xcchart* pszType)
	{
		return GetResourceHandle(MAKEINTRESOURCE(uiID), pszType);
	}

	// Get resource handle for a specific resource...
	inline HMODULE GetResourceHandle(xcchart* pszName, xcchart* pszType)
	{
		_XAssert(m_hModule || m_ModuleResource.IsValid());

		if (m_ModuleResource.IsValid())
		{
			return ::FindResource(m_ModuleResource.GetSafeType(), pszName, pszType) ? m_ModuleResource.GetSafeType() : m_hModule;
		}
		else
		{
			return m_hModule;
		}
	}

	// Get the module handle...
	inline HMODULE GetModuleHandle()
	{
		_XAssert(m_hModule);

		return m_hModule;
	}

	// Get the resource handle...
	inline HMODULE GetResourceHandle()
	{
		return m_ModuleResource.IsValid() ? m_ModuleResource.GetSafeType() : NULL;
	}

	// Various resource loading...
	_XApplication_LoadResource(Accelerators, HACCEL)
	_XApplication_LoadResource(Bitmap, HBITMAP)
	_XApplication_LoadResource(Cursor, HCURSOR)
	_XApplication_LoadResource(Icon, HICON)
	_XApplication_LoadResource(Menu, HMENU)

	// String loading...
	inline XString LoadString(xcuint32 uiID)
	{
		_XAssert(m_hModule || m_ModuleResource.IsValid());

		// Try fixed buffer first (to avoid wasting space in the heap)...
		XString sTemp;
		xchart sz[256] = {__XT('\0')};
		xint32 iSize = _XCountCharT(sz);
		xint32 iLength = LoadString(uiID, sz, iSize);
		if (iLength < iSize - 1)
		{
			sTemp = sz;

			return sTemp;
		}

		// Try buffer size of 512, then larger size until entire string is retrieved...
		do
		{
			iSize += 256;

			xchart* psz = sTemp.GetBuffer(iSize - 1);
			_XAssert(psz);
			if (!psz)
			{
				iLength = 0;

				break;
			}

			iLength = LoadString(uiID, psz, iSize);

		} while (iSize - iLength < 1);

		sTemp.ReleaseBuffer();

		return sTemp;
	}

	// Icon loading (variable size)...
	inline HICON LoadIconEx(xcuint32 uiID, xcuint32 uiCx = ::GetSystemMetrics(SM_CXSMICON), xcuint32 uiCy = ::GetSystemMetrics(SM_CYSMICON))
	{
		return LoadIconEx(MAKEINTRESOURCE(uiID), uiCx, uiCy);
	}

	// Icon loading (variable size)...
	inline HICON LoadIconEx(xcchart* pszName, xcuint32 uiCx = ::GetSystemMetrics(SM_CXSMICON), xcuint32 uiCy = ::GetSystemMetrics(SM_CYSMICON))
	{
		_XAssert(m_hModule || m_ModuleResource.IsValid());

		HANDLE hIcon = ::LoadImage(m_ModuleResource.GetSafeType() ? m_ModuleResource.GetSafeType() : m_hModule, pszName, IMAGE_ICON, uiCx, uiCy, 0);

		if (!hIcon)
		{
			hIcon = ::LoadIcon(NULL, pszName);
		}

		_XAssert(hIcon);

		return (HICON) hIcon;
	}

private:
// Internal functions...
	inline xresult GetResourceFileName(xcchart* pszFileName, xcchart* pszOEM, xcchart* pszLanguage, xchart** pszModule)
	{
		_XAssert(pszFileName && pszModule);
		if (!pszFileName || !pszModule)
			return xeInvalidArgument;
		XString s(pszFileName);

		if (pszOEM)
		{
			s += __XT(".");
			s += pszOEM;
		}

		if (pszLanguage)
		{
			s += __XT(".");
			s += pszLanguage;
		}

		s += _XResourceModuleExtension;

		return (*pszModule = s.Duplicate()) ? xsOK : xeOutOfMemory;
	}

	inline xint32 LoadString(xcuint32 uiID, xchart* pszBuffer, xcuint32 uiSize)
	{
		_XAssert(m_hModule || m_ModuleResource.IsValid());

		xint32 iLength = 0;

		if (m_ModuleResource.IsValid())
		{
			iLength = ::LoadString(m_ModuleResource.GetSafeType(), uiID, pszBuffer, uiSize);
		}

		if (iLength == 0)
		{
			iLength = ::LoadString(m_hModule, uiID, pszBuffer, uiSize);
		}

		_XAssert(iLength > 0);

		return iLength;
	}

private:
// Internal data...
	HMODULE m_hModule;
	XModule m_ModuleResource;
};



// #######################################################################################



// ***************************************************************************************
// Pointer to the one and only "XApp" or derived class object...
// ***************************************************************************************
extern XApplication* g_pApp;



// #######################################################################################



// ***************************************************************************************
// Function to access the application object. If this code asserts you have probably
//  forgotten to set "g_pApp" to the address of "XApplication" instance...
// ***************************************************************************************
inline static XApplication* GetApp()
{
	_XAssert(g_pApp);

	return g_pApp;
}



// #######################################################################################



// ***************************************************************************************
// XMessageLoop...
// ***************************************************************************************
class XMessageLoop : public CMessageLoop
{
public:
// Contstruction/Destruction...
	inline XMessageLoop()
	{
		m_iExitCode = 0;
	}

	inline ~XMessageLoop()
	{
	}

public:
// Operations...
	// Start message pump...
	inline xint32 Run()
	{
		xbool bDoIdle = true;
		xint32 iIdleCount = 0;
		xint32 iResult;

		while (1)
		{
			while (bDoIdle && !::PeekMessage(&m_msg, NULL, 0, 0, PM_NOREMOVE))
			{
				if (!OnIdle(iIdleCount++))
					bDoIdle = FALSE;
			}

			iResult = ::GetMessage(&m_msg, NULL, 0, 0);

			if (iResult == -1)
			{
				continue;
			}
			else if (!iResult || m_iExitCode)
			{
				break;
			}

			if (!PreTranslateMessage(&m_msg))
			{
				::TranslateMessage(&m_msg);
				::DispatchMessage(&m_msg);
			}

			if (IsIdleMessage(&m_msg))
			{
				bDoIdle = TRUE;
				iIdleCount = 0;
			}
		}

		return m_iExitCode ? (xint32) m_iExitCode : (xint32) m_msg.wParam;
	}

	// Exit the message pump...
	inline xvoid Exit(xcintptr iExitCode)
	{
		m_iExitCode = iExitCode;

		::PostThreadMessage(::GetCurrentThreadId(), WM_NULL, 0, 0L);
	}

private:
// Internal data...
	xintptr m_iExitCode;
};



// #######################################################################################



// ***************************************************************************************
// XPoint...
// ***************************************************************************************
class XPoint : public CPoint
{
public:
// Construction/Destruction...
	inline XPoint()
	{
		ZeroMemory(this, sizeof(*this));
	}

	inline XPoint(xcint32 iX, xcint32 iY) : CPoint(iX, iY)
	{
	}

	inline XPoint(const POINT& point) : CPoint(point)
	{
	}

	inline XPoint(const SIZE& size) : CPoint(size)
	{
	}

	inline XPoint(xcdword dwPoint) : CPoint(dwPoint)
	{
	}

	inline ~XPoint()
	{
	}
};



// #######################################################################################



// ***************************************************************************************
// XRect...
// ***************************************************************************************
class XRect : public CRect
{
public:
// Construction/Destruction...
	inline XRect()
	{
		ZeroMemory(this, sizeof(*this));
	}

	inline XRect(xcint32 iLeft, xcint32 iTop, xcint32 iRight, xcint32 iBottom) : CRect(iLeft, iTop, iRight, iBottom)
	{
	}

	inline XRect(const RECT& rc) : CRect(rc)
	{
	}

	inline XRect(const RECT* pRect) : CRect(pRect)
	{
	}

	inline XRect(POINT pt, SIZE size) : CRect(pt, size)
	{
	}

	inline XRect(POINT ptTopLeft, POINT ptBottomRight) : CRect(ptTopLeft, ptBottomRight)
	{
	}

	inline ~XRect()
	{
	}
};



// #######################################################################################



// ***************************************************************************************
// XSize...
// ***************************************************************************************
class XSize : public CSize
{
public:
	// Construction/Destruction...
	inline XSize()
	{
		ZeroMemory(this, sizeof(*this));
	}

	inline XSize(xcint32 iCx, xcint32 iCy) : CSize(iCx, iCy)
	{
	}

	inline XSize(SIZE size) : CSize(size)
	{
	}

	inline XSize(POINT pt) : CSize(pt)
	{
	}

	inline XSize(xcdword dwSize) : CSize(dwSize)
	{
	}

	inline ~XSize()
	{
	}
};



// #######################################################################################



// ***************************************************************************************
// XWindowBaseT: Window object extension base...
// ***************************************************************************************
template
<
	typename t_Base
>
class XWindowBaseT : public t_Base
{
public:
// Construction/Destruction...
	inline XWindowBaseT(HWND hWnd = NULL) : t_Base(hWnd)
	{
	}

	inline ~XWindowBaseT()
	{
	}

public:
// Operations...
	inline HWND GetSafeHwnd()
	{
		_XAssertWnd(m_hWnd);

		return m_hWnd;
	}

	inline bool SetWindowText(xcuint32 uiID)
	{
		_XAssert(uiID);
		if (!uiID)
			return false;

		XString sText = GetApp()->LoadString(uiID);

		return !!t_Base::SetWindowText(sText);
	}

	inline xbool SetWindowText(xcchart* pszText)
	{
		return !!t_Base::SetWindowText(pszText);
	}

	inline XString GetWindowText()
	{
		int iLength = GetWindowTextLength();

		XString sText;
		t_Base::GetWindowText(sText.GetBuffer(iLength), iLength + 1);
		
		sText.ReleaseBuffer();
		
		return sText;
	}

	inline xint32 GetWindowWidth()
	{
		XRect rc;
		GetWindowRect(&rc);

		return rc.Width();
	}

	inline xint32 GetWindowHeight()
	{
		XRect rc;
		GetWindowRect(&rc);

		return rc.Height();
	}

	inline xint32 GetClientWidth()
	{
		XRect rc;
		GetClientRect(&rc);

		return rc.Width();
	}

	inline xint32 GetClientHeight()
	{
		XRect rc;
		GetClientRect(&rc);

		return rc.Height();
	}
};

typedef XWindowBaseT<CWindow> XWindow;



// #######################################################################################



// ***************************************************************************************
// XAnimateCtrl...
// ***************************************************************************************
class XAnimateCtrl : public CAnimateCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XAnimateCtrl()
	{
	}

	inline ~XAnimateCtrl()
	{
	}

public:
// Operations...
	inline XAnimateCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XButton...
// ***************************************************************************************
class XButton : public CButtonT<XWindow>
{
public:
// Construction/Destruction...
	inline XButton()
	{
	}

	inline ~XButton()
	{
	}

public:
// Operations...
	inline XButton& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XComboBox...
// ***************************************************************************************
class XComboBox : public CComboBoxT<XWindow>
{
public:
// Construction/Destruction...
	inline XComboBox()
	{
	}

	inline ~XComboBox()
	{
	}

public:
// Operations...
	inline XComboBox& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}

	// Retrieve item text...
	XString GetItemText(xcint32 iItem = -1)
	{
		_XAssertWnd(m_hWnd);

		XString sText;
		xcint32 iLength = GetLBTextLen(iItem == -1 ? GetCurSel() : iItem);
		if (iLength != CB_ERR)
		{
			xchart* psz = sText.GetBufferSetLength(iLength);
			if (psz)
			{
				GetLBText(iItem, psz);

				sText.ReleaseBuffer();
			}
		}

		return sText;
	}
};



// #######################################################################################



// ***************************************************************************************
// XComboBoxEx...
// ***************************************************************************************
class XComboBoxEx : public CComboBoxExT<XWindow>
{
// Base class...
typedef CComboBoxExT<XWindow> _Base;

public:
// Construction/Destruction...
	inline XComboBoxEx()
	{
	}

	inline ~XComboBoxEx()
	{
	}

public:
// Operations...
	inline XComboBoxEx& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}

	// Retrieve the item count...
	inline xint32 GetItemCount() const
	{
		_XAssertWnd(m_hWnd);

		return GetCount();
	}

	// Retrieve item text...
	XString GetItemText(xcint32 iItem = -1)
	{
		_XAssertWnd(m_hWnd);

		XString sText;
		xcint32 iLength = GetLBTextLen(iItem == -1 ? GetCurSel() : iItem);
		if (iLength != CB_ERR)
		{
			xchart* psz = sText.GetBufferSetLength(iLength);
			if (psz)
			{
				GetLBText(iItem, psz);

				sText.ReleaseBuffer();
			}
		}

		return sText;
	}

	// Delete all items...
	inline xvoid DeleteAllItems()
	{
		_XAssertWnd(m_hWnd);

		xcint32 iCount = GetItemCount();

		SetRedraw(false);
		
		for (xint32 i = 0; i < iCount; i++)
		{
			DeleteItem(0);
		}

		SetRedraw();
	}

	inline xint32 InsertItem(xcchart* pszText, xcint32 iImage = -1, xclong lParam = 0, xcint32 iPos = -1)
	{
		_XAssert(IsWindow() && pszText);
		if (!pszText)
			return -1;

		COMBOBOXEXITEM cbi;
		ZeroMemory(&cbi, sizeof(cbi));
		cbi.mask = CBEIF_TEXT;
		cbi.pszText = (LPTSTR) pszText;
		cbi.iItem = iPos == -1 ? GetItemCount() : iPos;
		
		if (iImage != -1)
		{
			cbi.mask |= CBEIF_IMAGE | CBEIF_SELECTEDIMAGE;
			cbi.iImage = iImage;
			cbi.iSelectedImage = iImage;
		}

		if (lParam)
		{
			cbi.mask |= CBEIF_LPARAM;
			cbi.lParam = lParam;
		}

		return _Base::InsertItem(&cbi);
	}

	inline int InsertItem(xcuint32 uiID, xcint32 iImage = -1, xclong lParam = 0, xcint32 iPos = -1)
	{
		_XAssert(IsWindow() && uiID);
		if (!uiID)
			return -1;

		return InsertItem(GetApp()->LoadString(uiID), iImage, lParam, iPos);
	}
};



// #######################################################################################



// ***************************************************************************************
// XCommandBarCtrl...
// ***************************************************************************************
class XCommandBarCtrl : public CCommandBarCtrl
{
public:
// Construction/Destruction...
	inline XCommandBarCtrl()
	{
	}

	inline ~XCommandBarCtrl()
	{
	}

public:
// Operations...
	inline XCommandBarCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XDateTimePickerCtrl...
// ***************************************************************************************
class XDateTimePickerCtrl : public CDateTimePickerCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XDateTimePickerCtrl()
	{
	}

	inline ~XDateTimePickerCtrl()
	{
	}

public:
// Operations...
	inline XDateTimePickerCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XEdit...
// ***************************************************************************************
class XEdit : public CEditT<XWindow>
{
public:
// Construction/Destruction...
	inline XEdit()
	{
	}

	inline ~XEdit()
	{
	}

public:
// Operations...
	inline XEdit& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XHeaderCtrl...
// ***************************************************************************************
class XHeaderCtrl : public CHeaderCtrlT<XWindow>
{
// Base class...
typedef CHeaderCtrlT<XWindow> _Base;

public:
// Construction/Destruction...
	inline XHeaderCtrl()
	{
	}

	inline XHeaderCtrl(HWND hWnd) : _Base(hWnd)
	{
	}

	inline ~XHeaderCtrl()
	{
	}

public:
// Operations...
	inline XHeaderCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XHotKeyCtrl...
// ***************************************************************************************
class XHotKeyCtrl : public CHotKeyCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XHotKeyCtrl()
	{
	}

	inline ~XHotKeyCtrl()
	{
	}

public:
// Operations...
	inline XHotKeyCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XHyperLink...
// ***************************************************************************************
class XHyperLink : public CHyperLink
{
public:
// Construction/Destruction...
	inline XHyperLink()
	{
	}

	inline ~XHyperLink()
	{
	}

public:
// Operations...
	inline XHyperLink& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XIPAddressCtrl...
// ***************************************************************************************
class XIPAddressCtrl : public CIPAddressCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XIPAddressCtrl()
	{
	}

	inline ~XIPAddressCtrl()
	{
	}

public:
// Operations...
	inline XIPAddressCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XLinkCtrl...
// ***************************************************************************************
class XLinkCtrl : public CLinkCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XLinkCtrl()
	{
	}

	inline ~XLinkCtrl()
	{
	}

public:
// Operations...
	inline XLinkCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XListBox...
// ***************************************************************************************
class XListBox : public CListBoxT<XWindow>
{
public:
// Construction/Destruction...
	inline XListBox()
	{
	}

	inline ~XListBox()
	{
	}

public:
// Operations...
	inline XListBox& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XListCtrl...
// ***************************************************************************************
class XListCtrl : public CListViewCtrlT<XWindow>
{
// Base class
typedef CListViewCtrlT<XWindow> _Base;

public:
// Construction/Destruction...
	inline XListCtrl()
	{
	}

	inline ~XListCtrl()
	{
	}

public:
// Operations...
	inline XListCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}

	inline xint32 InsertColumn(xcint32 iColumn, xcchart* pszText, xcint32 iWidth = 0)
	{
		LVCOLUMN lvc;
		ZeroMemory(&lvc, sizeof(lvc));
		lvc.mask = LVCF_TEXT;
		lvc.pszText = (LPTSTR) pszText;

		if (iWidth != 0)
		{
			lvc.mask |= LVCF_WIDTH;
			lvc.cx = iWidth;
		}

		return _Base::InsertColumn(iColumn, &lvc);
	}

	inline xint32 InsertColumn(xcint32 iColumn, xcuint32 uiID, xcint32 iWidth = -1)
	{
		_XAssert(IsWindow() && uiID);
		if (!uiID)
			return -1;

		return InsertColumn(iColumn, GetApp()->LoadString(uiID), iWidth);
	}

	inline xbool InsertSubItem(xcint32 iItem, xcint32 iSubitem, xcchart* pszItem)
	{
		return !!SetItemText(iItem, iSubitem, pszItem);
	}

	inline XString GetItemText(xcint32 iItem, xcint32 iSubItem = 0) const
	{
		_XAssert(IsWindow());

		LVITEM lvi;
		ZeroMemory(&lvi, sizeof(lvi));
		lvi.iSubItem = iSubItem;

		XString sText;

		xint32 iResult = 0;
		for (xint32 iLength = 256; /* nothing */; iLength *= 2)
		{
			lvi.cchTextMax = iLength;
			lvi.pszText = sText.GetBufferSetLength(iLength);
			if(lvi.pszText == NULL)
			{
				iLength = 0;
				break;
			}

			iLength  = (xint32) ::SendMessage(m_hWnd, LVM_GETITEMTEXT, (WPARAM) iItem, (LPARAM) &lvi);
			if (iResult < iLength - 1)
				break;
		}

		sText.ReleaseBuffer();

		return sText;
	}

	inline int GetItemText(xcint32 iItem, xcint32 iSubItem, xchart* pszText, xcint32 iLength) const
	{
		return _Base::GetItemText(iItem, iSubItem, pszText, iLength);
	}

};



// #######################################################################################



// ***************************************************************************************
// XListCtrlSort...
// ***************************************************************************************
class XListCtrlSort : public CSortListViewCtrlImpl<XListCtrlSort, XListCtrl>
{
public:
// Construction/Destruction...
	inline XListCtrlSort()
	{
	}

	inline ~XListCtrlSort()
	{
	}

public:
// Operations...
	inline XListCtrlSort& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XListCtrlSortT...
// ***************************************************************************************
template
<
	typename t_Type,
	xcuint32 uiID
>
class XListCtrlSortT
{
// Message map...
BEGIN_MSG_MAP(XListCtrlSortT)
	MESSAGE_HANDLER(WM_CREATE, OnCreate)
	MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
	NOTIFY_HANDLER(uiID, LVN_COLUMNCLICK, OnColumnClick)
END_MSG_MAP()

public:
// Construction/Destruction...
	XListCtrlSortT()
	{
		m_lSort = 0;
	}

	~XListCtrlSortT()
	{
	}

protected:
// Message handler...
	// Window messages...
	inline LRESULT OnCreate(UINT uiMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;

		Update();

		return 0;
	}
	
	inline LRESULT OnInitDialog(UINT uiMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;

		Update();

		return 0;
	}

	// Notification messages...
	inline LRESULT OnColumnClick(int iCtrl, LPNMHDR pnmh, BOOL& bHandled)
	{
		bHandled = FALSE;

		LPNMLISTVIEW pnmlv = reinterpret_cast<LPNMLISTVIEW>(pnmh);

		Update(pnmlv->iSubItem);

		return 0;
	}

private:
// Internal functions...
	inline xvoid Update(xcint32 iColumn = 0)
	{
		XListCtrl& lc = static_cast<t_Type*>(this)->GetListCtrl();

		m_lSort = MAKELONG(iColumn, HIWORD(m_lSort) ^ 0xFFFF);

		lc.SetSelectedColumn(iColumn);

		lc.SortItemsEx(CompareProc, (LPARAM) this);
	}

	static xint32 CALLBACK CompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
	{
		XListCtrlSortImpl<t_Type, uiID>* pThis = reinterpret_cast<XListCtrlSortImpl<t_Type, uiID>* >(lParamSort);
		if (!pThis)
			return 0;

		XListCtrl& lc = static_cast<t_Type*>(pThis)->GetListCtrl();

		XString sText1 = lc.GetItemText((const int) lParam1, LOWORD(pThis->m_lSort));
		XString sText2 = lc.GetItemText((const int) lParam2, LOWORD(pThis->m_lSort));

		return sText1.Compare(sText2) * ((HIWORD(pThis->m_lSort) & 0xFFFF) ? 1 : -1);
	}

private:
// Internal data...
	long m_lSort;
};



// #######################################################################################



// ***************************************************************************************
// XMonthCalendarCtrl...
// ***************************************************************************************
class XMonthCalendarCtrl : public CMonthCalendarCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XMonthCalendarCtrl()
	{
	}

	inline ~XMonthCalendarCtrl()
	{
	}

public:
// Operations...
	inline XMonthCalendarCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XPagerCtrl...
// ***************************************************************************************
class XPagerCtrl : public CPagerCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XPagerCtrl()
	{
	}

	inline ~XPagerCtrl()
	{
	}

public:
// Operations...
	inline XPagerCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XProgressBarCtrl...
// ***************************************************************************************
class XProgressBarCtrl : public CProgressBarCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XProgressBarCtrl()
	{
	}

	inline ~XProgressBarCtrl()
	{
	}

public:
// Operations...
	inline XProgressBarCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XReBarCtrl...
// ***************************************************************************************
class XReBarCtrl : public CReBarCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XReBarCtrl(HWND hWnd = NULL)
	{
	}

	inline ~XReBarCtrl()
	{
	}

public:
// Operations...
	inline XReBarCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XRichEditCtrl...
// ***************************************************************************************
class XRichEditCtrl : public CRichEditCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XRichEditCtrl()
	{
	}

	inline ~XRichEditCtrl()
	{
	}

public:
// Operations...
	inline XRichEditCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XScrollBar...
// ***************************************************************************************
class XScrollBar : public CScrollBarT<XWindow>
{
public:
// Construction/Destruction...
	inline XScrollBar()
	{
	}

	inline ~XScrollBar()
	{
	}

public:
// Operations...
	inline XScrollBar& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XSplitterWindow...
// ***************************************************************************************
template
<
	bool bVertical
>
class XSplitterWindow : public CSplitterWindowImpl<XSplitterWindow<bVertical>, bVertical, XWindowBaseT<CWindow> >
{
public:
// Construction/Destruction...
	inline XSplitterWindow()
	{
	}

	inline ~XSplitterWindow()
	{
	}

public:
// Operations...
	inline XSplitterWindow& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};

// Vertical splitter window...
typedef XSplitterWindow<true>	XSplitterWindowV;
typedef XSplitterWindow<false>	XSplitterWindowH;



// #######################################################################################



// ***************************************************************************************
// XStatic...
// ***************************************************************************************
class XStatic : public CStaticT<XWindow>
{
public:
// Construction/Destruction...
	inline XStatic()
	{
	}

	inline ~XStatic()
	{
	}

public:
// Operations...
	inline XStatic& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XStatusBarCtrl...
// ***************************************************************************************
class XStatusBarCtrl : public CStatusBarCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XStatusBarCtrl()
	{
	}

	inline ~XStatusBarCtrl()
	{
	}

public:
// Operations...
	inline XStatusBarCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XTabCtrl...
// ***************************************************************************************
class XTabCtrl : public CTabCtrlT<XWindow>
{
// Base class...
typedef CTabCtrlT<XWindow> _Base;

public:
// Construction/Destruction...
	inline XTabCtrl()
	{
	}

	inline XTabCtrl(HWND hWnd) : _Base(hWnd)
	{
	}

	inline ~XTabCtrl()
	{
	}

public:
// Operations...
	inline XTabCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}

	inline DWORD GetItemData(xcuint32 uiItem)
	{
		TCITEM tci = {TCIF_PARAM, 0, 0, NULL, 0, 0, 0};

		SendMessage(TCM_GETITEM, (WPARAM) uiItem, (LPARAM) &tci);

		return (DWORD) tci.lParam;
	}

	inline xint32 InsertItem(xcuint32 uiItem, xcchart* pszText, LPARAM lParam = 0)
	{
		TCITEM tci;
		ZeroMemory(&tci, sizeof(tci));
		tci.mask = TCIF_TEXT | (lParam ? TCIF_PARAM : 0);
		tci.pszText = (LPTSTR) pszText;
		tci.lParam = lParam;

		return _Base::InsertItem(uiItem, &tci);
	}

	inline xint32 InsertItem(xcuint32 uiItem, xcuint32 uiID, LPARAM lParam = 0)
	{
		return InsertItem(uiItem, GetApp()->LoadString(uiID), lParam);
	}
};



// #######################################################################################



// ***************************************************************************************
// XToolBarCtrl...
// ***************************************************************************************
class XToolBarCtrl : public CToolBarCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XToolBarCtrl()
	{
	}

	inline ~XToolBarCtrl()
	{
	}

public:
// Operations...
	inline XToolBarCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XToolTipCtrl...
// ***************************************************************************************
class XToolTipCtrl : public CToolTipCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XToolTipCtrl()
	{
	}

	inline ~XToolTipCtrl()
	{
	}

public:
// Operations...
	inline XToolTipCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XTrackBarCtrl...
// ***************************************************************************************
class XTrackBarCtrl : public CTrackBarCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XTrackBarCtrl()
	{
	}

	inline ~XTrackBarCtrl()
	{
	}

public:
// Operations...
	inline XTrackBarCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XTreeCtrl...
// ***************************************************************************************
class XTreeCtrl : public CTreeViewCtrlT<XWindow>
{
// Base class...
typedef CTreeViewCtrlT<XWindow> _Base;

public:
// Construction/Destruction...
	inline XTreeCtrl()
	{
	}

	inline ~XTreeCtrl()
	{
	}

public:
// Operations...
	inline XTreeCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}

	inline HTREEITEM InsertItem(xcchart* pszText, xcint32 iImage, xcint32 iImageSelected, xclong lParam = 0, HTREEITEM hItemAfter = TVI_LAST, HTREEITEM hItemParent = TVI_ROOT)
	{
		_XAssert(pszText);
		if (!pszText)
			return NULL;

		TVINSERTSTRUCT tvi;
		ZeroMemory(&tvi, sizeof(tvi));
		tvi.hParent = hItemParent;
		tvi.hInsertAfter = hItemAfter;
		tvi.item.mask = TVIF_TEXT;
		tvi.item.pszText = (LPTSTR) pszText;
		
		if (lParam)
		{
			tvi.item.mask |= TVIF_PARAM;
			tvi.item.lParam = lParam;
		}

		if (iImage > 0)
		{
			tvi.item.mask |= TVIF_IMAGE;
			tvi.item.iImage = iImage;
		}

		if (iImageSelected > 0)
		{
			tvi.item.mask |= TVIF_SELECTEDIMAGE;
			tvi.item.iSelectedImage = iImageSelected;
		}

		return _Base::InsertItem(&tvi);
	}

	inline HTREEITEM InsertItem(xcuint32 uiID, xcint32 iImage, xcint32 iImageSelected, xclong lParam = 0, HTREEITEM hItemAfter = TVI_LAST, HTREEITEM hItemParent = TVI_ROOT)
	{
		_XAssert(uiID);
		if (!uiID)
			return NULL;

		XString sText = GetApp()->LoadString(uiID);

		return InsertItem(sText, iImage, iImageSelected, lParam, hItemAfter, hItemParent);
	}

	inline XString GetItemText(HTREEITEM hItem)
	{
		XString sText;

		CString sTemp;
		if (_Base::GetItemText(hItem, sTemp))
		{
			sText = sTemp;
		}

		return sText;
	}
};



// #######################################################################################



// ***************************************************************************************
// XUpDownCtrl...
// ***************************************************************************************
class XUpDownCtrl : public CUpDownCtrlT<XWindow>
{
public:
// Construction/Destruction...
	inline XUpDownCtrl()
	{
	}

	inline ~XUpDownCtrl()
	{
	}

public:
// Operations...
	inline XUpDownCtrl& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XWaitCursor...
// ***************************************************************************************
class XWaitCursor : public CWaitCursor
{
public:
// Construction/Destruction...
	inline XWaitCursor()
	{
	}

	inline ~XWaitCursor()
	{
	}
};



// #######################################################################################



// ***************************************************************************************
// XCustomCtrlT...
// ***************************************************************************************
template
<
	typename t_Type
>
class XCustomCtrlT : public CWindowImplBaseT<XWindow>
{
public:
// Construction/Destruction...
	inline XCustomCtrlT()
	{
	}

	inline ~XCustomCtrlT()
	{
	}

public:
// Operations...
	inline XCustomCtrlT<t_Type>& operator=(HWND hWnd)
	{
		m_hWnd = hWnd;

		return *this;
	}

	// Window class registration...
	inline static xresult RegisterClass(HINSTANCE hInstance)
	{
		WNDCLASSEX wcex;
		ZeroMemory(&wcex, sizeof(WNDCLASSEX));
		wcex.cbSize = sizeof(WNDCLASSEX); 
		wcex.style = CS_PARENTDC;
#ifdef _DEBUG
		wcex.hbrBackground = ::GetSysColorBrush(COLOR_INFOBK);
#else // _DEBUG
		wcex.hbrBackground = ::GetSysColorBrush(HOLLOW_BRUSH);
#endif
		wcex.lpfnWndProc = (WNDPROC) t_Type::WindowProcCustomCtrl;
		wcex.hInstance = hInstance;
		wcex.lpszClassName = t_Type::GetWndClassName();

		return ::RegisterClassEx(&wcex) ? S_OK : _XResultLastError;
	}

	// Window class unregistration...
	inline static xresult UnregisterClass(HINSTANCE hInstance)
	{
		return ::UnregisterClass(T::GetWndClassName(), hInstance) ? xsOK : _XResultLastError;
	}

	// Window procedure...
	static LRESULT CALLBACK WindowProcCustomCtrl(HWND hWnd, xuint32 uiMsg, xuintptr wParam, xlongptr lParam)
	{
		// We create the appropriate object here do default ATL/WTL window creation stuff...
		if (uiMsg == WM_NCCREATE)
		{
			t_Type* pCtrl = _xnew t_Type;
			if (pCtrl)
			{
				// HACK: Direct ATL/WTL relation...
				_AtlWinModule.AddCreateWndData(&(pCtrl->m_thunk.cd), pCtrl);

				t_Type::StartWindowProc(hWnd, uiMsg, wParam, lParam);
			}
		}

		return ::DefWindowProc(hWnd, uiMsg, wParam, lParam);
	}

protected:
// Message handler...
	inline virtual xvoid OnFinalMessage(HWND hWnd)
	{
		// This object is created dynamically in "WindowProcCustomCtrl()" therefore this is the place to destroy it properly...
		_xdelete this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XBrush...
// ***************************************************************************************
class XBrush : public CBrushT<true>
{
public:
// Construction/Destruction...
	inline XBrush()
	{
	}

	inline ~XBrush()
	{
	}

public:
// Operations...
	XBrush& operator=(HBRUSH hBrush)
	{
		Attach(hBrush);

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XFont...
// ***************************************************************************************
class XFont : public CFontT<true>
{
public:
// Construction/Destruction...
	inline XFont(HFONT hFont = NULL) : CFontT<true>(hFont)
	{
	}

	inline ~XFont()
	{
	}

public:
// Operations...
	inline XFont& operator=(HFONT hFont)
	{
		Attach(hFont);

		return *this;
	}

	inline static HFONT GetDefaultFont()
	{
		return AtlGetDefaultGuiFont();
	}

	inline static HFONT GetDefaultFontBold()
	{
		static XFont font;

		if (font)
			return font;

		LOGFONT lf;
		ZeroMemory(&lf, sizeof(lf));
		if (::GetObject(GetDefaultFont(), sizeof(lf), &lf))
		{
			lf.lfWeight = FW_BOLD;
			font.CreateFontIndirect(&lf);
		}

		return font ? font : (font.Attach(AtlCreateBoldFont()), font);
	}
};



// #######################################################################################



// ***************************************************************************************
// XIcon: Object handling icons...
// ***************************************************************************************
class XIcon : public CIcon
{
// Base class...
typedef CIcon _Base;

public:
// Construction/Destruction...
	inline XIcon()
	{
	}

	inline ~XIcon()
	{
	}

public:
// Operations...
	inline XIcon& operator=(HICON hIcon)
	{
		_Base::operator=(hIcon);

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XBitmap: Object handling bitmaps...
// ***************************************************************************************
class XBitmap : public CBitmap
{
// Base class...
typedef CBitmap _Base;

public:
// Construction/Destruction...
	inline XBitmap()
	{
	}

	inline ~XBitmap()
	{
	}

public:
// Operations...
	inline XBitmap& operator=(HBITMAP hBmp)
	{
		_Base::operator=(hBmp);

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XDCT...
// ***************************************************************************************
template
<
	xbool t_bManaged = true
>
class XDCT : public CDCT<t_bManaged>
{
public:
// Construction/Destruction...
	inline XDCT()
	{
	}

	inline XDCT(HDC hDC) : CDCT<t_bManaged>(hDC)
	{
	}

	inline ~XDCT()
	{
	}
};

typedef XDCT<true> XDC;



// #######################################################################################



// ***************************************************************************************
// XMemoryDC: Encapsulates a memory DC for fast and flicker free drawing...
// ***************************************************************************************
class XDCMemory : public XDC
{
public:
// Construction/Destruction...
	// Pass the original DC and if appropriate a clipping rectangle...
	inline XDCMemory(HDC hDC, const RECT* pRect = NULL)
	{
		_XAssert(hDC);

		m_dcScreen = hDC;

		if (pRect)
		{
			m_rcClip = *pRect;
		}
		else
		{
			m_dcScreen.GetClipBox(&m_rcClip);
		}
	
		CreateCompatibleDC(hDC);

		m_dcScreen.LPtoDP(&m_rcClip);

		m_bmpDC.CreateCompatibleBitmap(m_dcScreen, m_rcClip.Width(), m_rcClip.Height());

		m_bmpScreen = SelectBitmap(m_bmpDC);

		SetMapMode(m_dcScreen.GetMapMode());

		XSize sizeWindowExt;
		m_dcScreen.GetWindowExt(&sizeWindowExt);
		SetWindowExt(sizeWindowExt);

		XSize sizeViewportExt;
		m_dcScreen.GetViewportExt(&sizeViewportExt);
		SetViewportExt(sizeViewportExt);

		m_dcScreen.DPtoLP(&m_rcClip);

		SetWindowOrg(m_rcClip.TopLeft());

		FillSolidRect(&m_rcClip, m_dcScreen.GetBkColor());
	}

	inline ~XDCMemory()
	{
		SelectBitmap(m_bmpScreen);
	}

public:
// Operations..
	// Copies the content of the memory DC back to the corresponding screen DC...
	inline xvoid CopyToScreen()
	{
		m_dcScreen.BitBlt(m_rcClip.left, m_rcClip.top, m_rcClip.Width(), m_rcClip.Height(), m_hDC, m_rcClip.left, m_rcClip.top, SRCCOPY);
	}

	inline XBitmap& GetBitmap()
	{
		return m_bmpDC;
	}

	// Allow usage as a pointer...  
	inline XDCMemory* operator->() 
	{
		return this;
	}       

	// Allow usage as a pointer...
	inline operator XDCMemory*() 
	{
		return this;
	}

private:
// Internal data...
	XDCT<false> m_dcScreen;
	XRect m_rcClip;
    XBitmap m_bmpDC;
    XBitmap m_bmpScreen;
};



// #######################################################################################



// ***************************************************************************************
// XPaintDC...
// ***************************************************************************************
class XPaintDC : public CPaintDC
{
public:
// Construction/Destruction...
	inline XPaintDC(HWND hWnd) : CPaintDC(hWnd)
	{
	}

	inline ~XPaintDC()
	{
	}
};



// #######################################################################################



// ***************************************************************************************
// XImageList...
// ***************************************************************************************
class XImageList : public CImageList
{
// Base class...
typedef CImageList _Base;
public:
// Construction/Destruction...
	inline XImageList(HIMAGELIST hImageList = NULL) : CImageList(hImageList)
	{
	}

	inline ~XImageList()
	{
	}

public:
// Operations...
	inline XImageList& operator=(HIMAGELIST hImageList)
	{
		CImageList::operator=(hImageList);

		return *this;
	}

	inline xbool Create(HWND hWnd, xcuint32 iCx = ::GetSystemMetrics(SM_CXSMICON), xcuint32 iCy = ::GetSystemMetrics(SM_CYSMICON))
	{
		return !!_Base::Create(iCx, iCy, GetFlags(hWnd), 0, 1);
	}

	inline static xuint32 GetFlags(HWND hWnd)
	{
		// The minimum quality supported is 8 bit per pixel...
		xuint32 uiFlags = ILC_MASK;

		HDC hDC = ::GetDC(hWnd);
		_XAssert(hDC);
		if (!hDC)
			return uiFlags | ILC_COLOR8;

		// Get the number of bits per pixel used by the display driver...
		xint32 iBit = ::GetDeviceCaps(hDC, BITSPIXEL);
		::ReleaseDC(hWnd, hDC);
		_XAssert(iBit);
		if (!iBit)
			return uiFlags | ILC_COLOR8;

		xdword dwMajor = 0;
		xdword dwMinor = 0;
		if (_XFailed(AtlGetCommCtrlVersion(&dwMajor, &dwMinor)))
			return uiFlags | ILC_COLOR8;

		// Determine the appropriate creation flags...
		if (iBit <= 4)
		{
			uiFlags |= ILC_COLOR4;
		}
		else if (iBit > 4 && iBit <= 8)
		{
			uiFlags |= ILC_COLOR8;
		}
		else if (iBit > 8 && iBit <= 16)
		{
			uiFlags |= ILC_COLOR16;
		}
		else if (iBit > 16 && iBit <= 24)
		{
			uiFlags |= ILC_COLOR24;
		}
		else if (iBit > 24 && iBit <= 32 && dwMajor >= 6)
		{
			uiFlags |= ILC_COLOR32;
		}
		else if (iBit > 24 && iBit <= 32 && dwMajor <= 5)
		{
			uiFlags |= ILC_COLOR16;
		}
		else
		{
			_XAssert(false);

			uiFlags |= ILC_COLOR16;
		}

		return uiFlags;
	}

	inline xint32 AddIconEx(xcchart* pszIcon, xcuint32 uiCx = ::GetSystemMetrics(SM_CXSMICON), xcuint32 uiCy = ::GetSystemMetrics(SM_CYSMICON))
	{
		_XAssert(pszIcon);
		if (!pszIcon)
			return -1;

		HICON hIcon = GetApp()->LoadIconEx(pszIcon, uiCx, uiCy);
		_XAssert(hIcon);
		if (!hIcon)
			return -1;

		xcint32 iResult = _Base::AddIcon(hIcon);

		::DestroyIcon(hIcon);

		return iResult;
	}

	inline xint32 AddIconEx(xcuint32 uiID, xcuint32 uiCx = ::GetSystemMetrics(SM_CXSMICON), xcuint32 uiCy = ::GetSystemMetrics(SM_CYSMICON))
	{
		return AddIconEx(MAKEINTRESOURCE(uiID), uiCx, uiCy);
	}

	inline xint32 AddIconEx(xcchart* pszFileName, xcbool bSmall)
	{
		_XAssert(pszFileName);
		if (!pszFileName)
			return -1;

		SHFILEINFO fi;
		_XMemoryZero(&fi, sizeof(SHFILEINFO));
		if (!::SHGetFileInfo(pszFileName, 0, &fi, sizeof(SHFILEINFO), SHGFI_ICON | (bSmall ? SHGFI_SMALLICON : SHGFI_LARGEICON)))
			return -1;

		xcint32 iIcon = _Base::AddIcon(fi.hIcon);

		::DestroyIcon(fi.hIcon);

		return iIcon;
	}
};



// #######################################################################################



// ***************************************************************************************
// XTheme: Theme API specific...
// ***************************************************************************************
class XTheme : public CTheme
{
public:
// Construction/Destruction...
	inline XTheme()
	{
	}

	inline ~XTheme()
	{
	}

public:
// Operations...
	inline static xbool IsAppThemed()
	{
		if (s_iIsAppThemed == -1)
		{
			CStaticDataInitCriticalSectionLock lock;
			if (_XFailed(lock.Lock()))
			{
				_XAssert(false);				
				return false;
			}

			typedef BOOL (STDAPICALLTYPE *XTheme_IsAppThemed)();
			HMODULE hThemeDll = ::LoadLibrary(_T("uxtheme.dll"));
			if (!hThemeDll)
				return false;

			XTheme_IsAppThemed pfn = (XTheme_IsAppThemed) ::GetProcAddress(hThemeDll, "IsAppThemed");
			s_iIsAppThemed = pfn ? pfn() : 0;

			::FreeLibrary(hThemeDll);

			lock.Unlock();
		}

		return (s_iIsAppThemed == 1);
	}

	static bool EnableThemeDialogTexture(HWND hWnd)
	{
		_XAssertWnd(hWnd);
		if (!hWnd || !::IsWindow(hWnd))
			return false;

		if (!IsAppThemed())
			return false;

		typedef BOOL (STDAPICALLTYPE *XTheme_EnableThemeDialogTexture)(HWND hWnd, DWORD dwFlags);
		HMODULE hThemeDll = ::LoadLibrary(_T("uxtheme.dll"));
		if (!hThemeDll)
			return false;

		XTheme_EnableThemeDialogTexture pfn = (XTheme_EnableThemeDialogTexture) ::GetProcAddress(hThemeDll, "EnableThemeDialogTexture");
		bool bResult = SUCCEEDED(pfn(hWnd, ETDT_ENABLETAB));
		
		::FreeLibrary(hThemeDll);

		return bResult;
	}

private:
// Internal data...
	static xint32 s_iIsAppThemed;
};

__declspec(selectany) xint32 XTheme::s_iIsAppThemed = -1;



// #######################################################################################



// ***************************************************************************************
// XDataExchangeT: Object handling data exchange...
// ***************************************************************************************
template
<
	class t_Type
>
class XDataExchangeT : public CWinDataExchange<t_Type>
{
public:
// Construction/Destruction...
	inline XDataExchangeT()
	{
	}

	inline ~XDataExchangeT()
	{
	}

public:
// Operations...
	xbool DDX_Text(xcuint32 uiID, XString& sText, xcint32 iSize, const BOOL bSave, const BOOL bValidate = false, xcint32 iLength = 0)
	{
		t_Type* pType = static_cast<t_Type*>(this);
		
		xbool bSuccess = true;

		if (bSave)
		{
			HWND hWndCtrl = pType->GetDlgItem(uiID);
			xint32 iLengthText = ::GetWindowTextLength(hWndCtrl);
			xint32 iLengthResult = -1;
			xchart* psz = sText.GetBufferSetLength(iLengthText);
			if (psz)
			{
				iLengthResult = ::GetWindowText(hWndCtrl, psz, iLengthText + 1);
				sText.ReleaseBuffer();
			}
			
			if (iLengthResult < iLengthText)
			{
				bSuccess = false;
			}
		}
		else
		{
			bSuccess = !!pType->SetDlgItemText(uiID, sText);
		}

		if (!bSuccess)
		{
			pType->OnDataExchangeError(uiID, bSave);
		}
		else if (bSave && bValidate)
		{
			_XAssert(iLength > 0);

			if (sText.GetLength() > (xuint32) iLength)
			{
				_XData data = {ddxDataText};
				data.textData.nLength = sText.GetLength();
				data.textData.nMaxLength = iLength;
				pType->OnDataValidateError(uiID, bSave, data);
				bSuccess = false;
			}
		}

		return bSuccess;
	}
};



/*
// ***************************************************************************************
// XAutoComplete: This object implements custom auto-completion...
// ***************************************************************************************
class XAutoComplete : public XRefCount<IXEnumString>
{
// Interface map...
XInterfaceMap_Begin
	XInterfaceMap_Entry(IXEnumString)
XInterfaceMap_End

public:
// Construction/Destruction...
	inline XAutoComplete()
	{
		m_ulCurrent = 0;
	}

	inline ~XAutoComplete()
	{
	}

public:
// Operations...
	// "IAutoComplete" bind and	unbind...
	inline xresult Bind(HWND hWndEdit, xcdword dwOptions = 0, xcchart* pszFormat = NULL)
	{
		_XAssertWnd(hWndEdit);
		if (!::IsWindow(hWndEdit))
			return xeInvalidArgument;

		HRESULT hr = m_pAutoComplete.CoCreateInstance(CLSID_AutoComplete);
		if (FAILED(hr))
			return hr;

		if (dwOptions)
		{
			CComPtr<IAutoComplete2> pAutoComplete2;
			if (FAILED(hr = m_pAutoComplete->QueryInterface(IID_IAutoComplete2, (void**) &pAutoComplete2)) ||
				FAILED(hr = pAutoComplete2->SetOptions(dwOptions)))
				return hr;
		}

		return m_pAutoComplete->Init(hWndEdit, this, NULL, CComBSTR(pszFormat));
	}

	inline HRESULT Unbind()
	{
		if (m_pAutoComplete)
		{
			m_pAutoComplete.Release();
		}

		return S_OK;
	}

	// Data persistence...
	inline HRESULT Load(LPCTSTR pszKey, LPCTSTR pszValue, HKEY hKey = HKEY_CURRENT_USER)
	{
		_XAssert(pszKey && pszValue && hKey);
		if (!pszKey || !pszValue || !hKey)
			return E_INVALIDARG;

		XRegistry	reg;
		XMultiString ms;
		HRESULT hr = reg.Open(hKey, pszKey);
		if (FAILED(hr) ||
			FAILED(hr = reg.ReadMultiString(pszValue, ms)))
			return hr;

		m_arrData.RemoveAll();
		m_arrData.Append(ms);

		return hr;
	}

	inline HRESULT Save(LPCTSTR pszKey, LPCTSTR pszValue, HKEY hKey = HKEY_CURRENT_USER)
	{
		_XAssert(pszKey && pszValue && hKey);
		if (!pszKey || !pszValue || !hKey)
			return E_INVALIDARG;

		XMultiString ms;
		ms.Append(m_arrData);

		XRegistry	reg;
		HRESULT hr = reg.CreateKey(hKey, pszKey);
		if (FAILED(hr))
			return hr;

		return reg.WriteMultiString(pszValue, ms);
	}

	// Add strings...
	inline HRESULT Add(LPCTSTR pszString, const bool bMulti = false)
	{
		_XAssert(pszString);
		if (!pszString)
			return E_INVALIDARG;

		if (bMulti)
		{
			XMultiString ms(pszString);
			Add(ms);
		}
		else
		{
			m_arrData.Add(pszString);
		}

		return S_OK;
	}

	inline HRESULT Add(XArray<XString>& arrString)
	{
		m_arrData.Append(arrString);

		return S_OK;
	}

public:
// Overridables...
	// *** IXEnumString...
	inline STDMETHOD(Next)(ULONG ulCount, LPOLESTR* pwsz, PULONG pulCountSupplied)
	{
		_XAssert(pwsz);
		if (!pwsz)
			return E_INVALIDARG;

		if (!ulCount)
		{
			ulCount = 1;
		}

		for (ULONG ul = 0; ul < ulCount; ul++)
		{
			if (m_ulCurrent == (ULONG) m_arrData.GetSize())
				break;

			pwsz[ul] = (LPWSTR) ::CoTaskMemAlloc((ULONG) sizeof(WCHAR) * (m_arrData[m_ulCurrent].GetLength() + 1));
			if (!pwsz[ul])
				return E_OUTOFMEMORY;

#ifdef UNICODE
			wcscpy(pwsz[ul], m_arrData[m_ulCurrent]);
#else // UNICODE
			_XConversion;
			wcscpy(pwsz[ul], _CexA2W(m_arrData[m_ulCurrent]));
#endif

			if (pulCountSupplied)
			{
				*pulCountSupplied++;
			}

			m_ulCurrent++;
		}

		return (ul == ulCount) ? S_OK : S_FALSE;
	}

	inline STDMETHOD(Skip)(ULONG ulCount)
	{
		m_ulCurrent += ulCount;

		if (m_ulCurrent > (ULONG) m_arrData.GetSize())
		{
			m_ulCurrent = 0;
		}

		return S_OK;
	}

	inline STDMETHOD(Reset)()
	{
		m_ulCurrent = 0;

		return S_OK;
	}

	inline STDMETHOD(Clone)(IEnumString** pEnumString)
	{
		_XAssert(pEnumString);
		if (!pEnumString)
			return E_INVALIDARG;

		// Create new object...
		XAutoComplete* p = new XAutoComplete();
		if (!p)
			return E_OUTOFMEMORY;

		// Copy original content...
		*p = *this;	

		*pEnumString = p;

		return S_OK;
	}

private:
// Internal data...
	CComPtr<IAutoComplete> m_pAutoComplete;
	XArray<XString> m_arrData;
	ULONG m_ulCurrent;
};
*/



// #######################################################################################



// ***************************************************************************************
// DragDropHelper definitions...
// ***************************************************************************************
extern "C" const GUID __declspec(selectany) CLSID_DragDropHelper = 
{ 0x4657278a, 0x411b, 0x11d2, { 0x83, 0x9a, 0x0, 0xc0, 0x4f, 0xd9, 0x18, 0xd0 }};

extern "C" const GUID __declspec(selectany) IID_IDropTargetHelper = 
{ 0x4657278b, 0x411b, 0x11d2, { 0x83, 0x9a, 0x0, 0xc0, 0x4f, 0xd9, 0x18, 0xd0 }};



// #######################################################################################



// ***************************************************************************************
// XDropTargetImpl: Object implemeting and handling drag 'n drop stuff...
// ***************************************************************************************
template
<
	typename t_Type
>
class XDropTargetImpl : public IDropTarget
{
// Message map...
BEGIN_MSG_MAP(XDropTargetImpl)
	MESSAGE_HANDLER(WM_CREATE, OnCreate)
	MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
END_MSG_MAP()

public:
// Construction/Destruction...
	inline XDropTargetImpl()
	{
		m_pDropTargetHelper = NULL;
		m_dwDropEffect = DROPEFFECT_COPY;
		m_bValid = false;
	}

	inline ~XDropTargetImpl()
	{
		UnregisterDropTarget();
	}

public:
// Operations...
	// Register object...
	inline HRESULT RegisterDropTarget()
	{
		return ::RegisterDragDrop(static_cast<t_Type*>(this)->GetSafeHwnd(), this);
	}

	// Unregister object...
	inline HRESULT UnregisterDropTarget()
	{
		if (m_pDropTargetHelper)
		{
			m_pDropTargetHelper->Release();
			m_pDropTargetHelper = NULL;
		}

		return ::RevokeDragDrop(static_cast<t_Type*>(this)->GetSafeHwnd());
	}

	// Registers a clipboard format. Can be called multiple times...
	inline xvoid RegisterFormat(xcuint32 uiClipboardFormat)
	{
		m_arrFormat.Add(uiClipboardFormat);

		// If someone registers Shell's clipboard format "CF_HDROP" we check if we can get an "IDropTargetHelper" interface to provide
		//  better visual feedback...
		if (uiClipboardFormat == CF_HDROP && !m_pDropTargetHelper)
		{
			::CoCreateInstance(CLSID_DragDropHelper, NULL, CLSCTX_INPROC_SERVER, IID_IDropTargetHelper,  (void**) &m_pDropTargetHelper);
		}
	}

	// Set the drop effect for feedback. Default is "DROPEFFECT_COPY"...
	inline xvoid SetDropEffect(xcdword dwDropEffect)
	{
		m_dwDropEffect = dwDropEffect;
	}

protected:
// Message handler...
	inline LRESULT OnCreate(UINT uiMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;

		RegisterDropTarget();

		return 0;
	}

	inline LRESULT OnDestroy(UINT uiMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;

		UnregisterDropTarget();

		return 0;
	}
	
protected:
// Overridables...
	//  Nearly everything is done by the implementations below. You probably only need to override "Drop()" in your derived object...
	inline STDMETHOD(DragEnter)(IDataObject* pDataObject, DWORD grfKeyState, POINTL pt, DWORD* pdwEffect)
	{
		*pdwEffect = (m_bValid = IsDataObjectValid(pDataObject)) && (*pdwEffect & m_dwDropEffect) ? m_dwDropEffect : DROPEFFECT_NONE;
		if (m_pDropTargetHelper)
		{
			m_pDropTargetHelper->DragEnter(static_cast<T*>(this)->GetSafeHwnd(), pDataObject, (LPPOINT) &pt, *pdwEffect);
		}

		return S_OK;
	}

	inline STDMETHOD(DragOver)(DWORD grfKeyState, POINTL pt, DWORD* pdwEffect)
	{
		*pdwEffect = m_bValid && (*pdwEffect & m_dwDropEffect) ? m_dwDropEffect : DROPEFFECT_NONE;
		if (m_pDropTargetHelper)
		{
			m_pDropTargetHelper->DragOver((LPPOINT) &pt, *pdwEffect);
		}

		return S_OK;
	}

	inline STDMETHOD(DragLeave)()
	{
		if (m_pDropTargetHelper)
		{
			m_pDropTargetHelper->DragLeave();
		}

		return S_OK;
	}

	inline STDMETHOD(Drop)(IDataObject* pDataObject, DWORD grfKeyState, POINTL pt, DWORD* pdwEffect)
	{
		*pdwEffect = m_bValid && (*pdwEffect & m_dwDropEffect) ? m_dwDropEffect : DROPEFFECT_NONE;
		if (m_pDropTargetHelper)
		{
			m_pDropTargetHelper->Drop(pDataObject, (LPPOINT) &pt, m_dwDropEffect);
		}

		return S_OK;
	}

	// Can be overridden to perform custom validation...
	inline virtual bool IsDataObjectValid(IDataObject* pDataObject) const
	{
		_XAssert(pDataObject);
		if (!pDataObject)
			return false;

		FORMATETC fe;	
		for (xuint32 ui = 0; ui < m_arrFormat.GetSize(); ui++)
		{
			ZeroMemory(&fe, sizeof(FORMATETC));
			fe.cfFormat = m_arrFormat[ui];
			fe.ptd = NULL;
			fe.dwAspect = DVASPECT_CONTENT;
			fe.lindex = -1;
			fe.tymed = ~0u;

			if (SUCCEEDED(pDataObject->QueryGetData(&fe)))
				return true;
		}

		return false;
	}

private:
// Internal data...
	IDropTargetHelper* m_pDropTargetHelper;
	XArrayT<xuint32> m_arrFormat;
	xdword m_dwDropEffect;
	xbool m_bValid;
};



// #######################################################################################



// ***************************************************************************************
// XDialogResizeT: Object handling data exchange...
// ***************************************************************************************
template
<
	typename t_Type
>
class XDialogResizeT : public CDialogResize<t_Type>
{
public:
// Construction/Destruction...
	inline XDialogResizeT()
	{
	}

	inline ~XDialogResizeT()
	{
	}
};



// #######################################################################################



// ***************************************************************************************
// XMenu: Object handling menus...
// ***************************************************************************************
class XMenu : public CMenu
{
public:
// Construction/Destruction...
	inline XMenu()
	{
	}

	inline ~XMenu()
	{
	}
};



// #######################################################################################



// ***************************************************************************************
// XFileDialog...
// ***************************************************************************************
class XFileDialog : public CFileDialog
{
public:
// Construction/Destruction...
	inline XFileDialog(bool bOpen,
					   xcchart* pszExtension = NULL,
					   xcchart* pszFileName = NULL,
					   xcdword dwFlags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
					   xcchart* pszFilter = NULL)
					   : CFileDialog(bOpen, pszExtension, pszFileName, dwFlags, pszFilter)
	{
	}

	inline ~XFileDialog()
	{
	}
};



// #######################################################################################



// ***************************************************************************************
// XDialogImpl...
// ***************************************************************************************
template
<
	typename t_Type,
	typename t_Base = XWindow
>
class XDialogT : public CDialogImpl<t_Type, t_Base>
{
// Base class...
typedef CDialogImpl<t_Type, t_Base> _Base;

// Message map...
BEGIN_MSG_MAP(XDialogT)
	COMMAND_HANDLER(IDOK, BN_CLICKED, OnOK)
	COMMAND_HANDLER(IDCANCEL, BN_CLICKED, OnCancel)
END_MSG_MAP()

public:
// Construction/Destruction...
	inline XDialogT()
	{
	}

	inline ~XDialogT()
	{
	}

public:
// Operations...

protected:
// Overridables...
	inline virtual LRESULT OnOK(WORD wNotifyCode, WORD wID, HWND hWndCtrl, BOOL& bHandled)
	{
		EndDialog(IDOK);

		return 0;
	}

	inline virtual LRESULT OnCancel(WORD wNotifyCode, WORD wID, HWND hWndCtrl, BOOL& bHandled)
	{
		EndDialog(IDCANCEL);

		return 0;
	}
};



// #######################################################################################



// ***************************************************************************************
// Style definitions...
// ***************************************************************************************
#define XFrameWindowT_ReBarStyle     (WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | RBS_VARHEIGHT | RBS_BANDBORDERS | RBS_AUTOSIZE | CCS_NODIVIDER | WS_BORDER)
#define XFrameWindowT_ReBarBandStyle (RBBS_CHILDEDGE | RBBS_NOGRIPPER | RBBS_USECHEVRON | RBBS_BREAK)



// #######################################################################################



// ***************************************************************************************
// XFrameWindowT...
// ***************************************************************************************
template
<
	typename t_Type,
	typename t_Base = XWindow
>
class XFrameWindowT : public CFrameWindowImpl<t_Type, t_Base>
{
public:
// Construction/Destruction...
	inline XFrameWindowT()
	{
	}

	inline ~XFrameWindowT()
	{
	}

public:
// Operations...
	static xbool AddSimpleReBarBandCtrl(HWND hWndReBar, HWND hWndBand, xcuint32 uiStyle = XFrameWindowT_ReBarBandStyle, xcchart* pszTitle = NULL)
	{
		_XAssertWnd(hWndReBar);
#ifdef _DEBUG
		// Check if this is really a rebar...
		{
			xchart szClassName[sizeof(REBARCLASSNAME)] = {_T('\0')};

			::GetClassName(hWndReBar, szClassName, sizeof(REBARCLASSNAME));

			_XAssert(_tcsicmp(szClassName, REBARCLASSNAME) == 0);
		}
#endif //_DEBUG
		_XAssertWnd(hWndReBar);

		// Get number of buttons on the toolbar...
		xcint32 iButtonCount = (xcint32) ::SendMessage(hWndBand, TB_BUTTONCOUNT, 0, 0L);

		// Set band info structure...
		REBARBANDINFO rbBand = {0};
		rbBand.cbSize = sizeof(REBARBANDINFO);
#if (_WIN32_IE >= 0x0400)
		rbBand.fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE | RBBIM_IDEALSIZE;
#else
		rbBand.fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE;
#endif //!(_WIN32_IE >= 0x0400)
		if (pszTitle)
		{
			rbBand.fMask |= RBBIM_TEXT;
		}

		rbBand.fStyle = uiStyle;
		rbBand.lpText = const_cast<LPTSTR>(pszTitle);
		rbBand.hwndChild = hWndBand;
		rbBand.wID = ATL_IDW_BAND_FIRST + (int) ::SendMessage(hWndReBar, RB_GETBANDCOUNT, 0, 0L);

		// Calculate the size of the band...
		BOOL bResult = FALSE;
		XRect rcTemp;
		if (iButtonCount > 0)
		{
			bResult = (BOOL) ::SendMessage(hWndBand, TB_GETITEMRECT, iButtonCount - 1, (LPARAM) &rcTemp);
			_XAssert(bResult);
			rbBand.cx = rcTemp.right;
			rbBand.cyMinChild = rcTemp.Height();
			if (pszTitle)
			{
				bResult = (BOOL) ::SendMessage(hWndBand, TB_GETITEMRECT, 0, (LPARAM) &rcTemp);
				_XAssert(bResult);
				rbBand.cxMinChild = rcTemp.right;
			}
			else
			{
				rbBand.cxMinChild = 0;
			}
		}
		else
		{
			// No buttons, either not a toolbar or really has no buttons...
			bResult = ::GetWindowRect(hWndBand, &rcTemp);
			_XAssert(bResult);
			rbBand.cx = rcTemp.Width();
			rbBand.cxMinChild = 0;
			rbBand.cyMinChild = rcTemp.Height();
		}

		rbBand.cxIdeal = rbBand.cx;

		// Add the band
		LRESULT lResult = ::SendMessage(hWndReBar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbBand);
		if (!lResult)
		{
			return false;
		}

#if (_WIN32_IE >= 0x0501)
		DWORD dwExStyle = (DWORD) ::SendMessage(hWndBand, TB_GETEXTENDEDSTYLE, 0, 0L);
		::SendMessage(hWndBand, TB_SETEXTENDEDSTYLE, 0, dwExStyle | TBSTYLE_EX_HIDECLIPPEDBUTTONS);
#endif //(_WIN32_IE >= 0x0501)

		return true;
	}

	inline xbool AddSimpleReBarBand(HWND hWndBand, const UINT uiStyle, LPCTSTR pszTitle = NULL)
	{
		_XAssertWnd(m_hWndToolBar);
		_XAssertWnd(hWndBand);

		return AddSimpleReBarBandCtrl(m_hWndToolBar, hWndBand, uiStyle, pszTitle);
	}
};



// #######################################################################################



// ***************************************************************************************
// XMessageBox: Use static "Show()" functions to operate in "::MessageBox()" API mode
//  or create an object and call "DoModal()" if you like to use enhanced features...
// ***************************************************************************************
class XMessageBox
{
public:
// Operations...
	inline static xint32 Show(xcchart* pszText, xcchart* pszCaption = NULL, xcuint32 uiFlags = MB_OK | MB_ICONEXCLAMATION, HWND hWndParent = ::GetActiveWindow())
	{
		XString sCaption;

		if (pszCaption)
		{
			sCaption = pszCaption;
		}
		else if (::IsWindow(hWndParent))
		{
			XWindow wnd(hWndParent);
			sCaption = wnd.GetWindowText();
			wnd.Detach();
		}

		// Initiate centering hook...
		s_hHook = ::SetWindowsHookEx(WH_CBT, _CBTProc, XModuleInformation::GetModule(), ::GetCurrentThreadId());

		return ::MessageBox(hWndParent, pszText, sCaption.GetLength() ? (xcchart*) sCaption : NULL, uiFlags);
	}

	inline static int Show(xcuint32 uiText, xcuint32 uiCaption = 0, xcuint32 uiFlags = MB_OK | MB_ICONEXCLAMATION, HWND hWndParent = ::GetActiveWindow())
	{
		XString sText = GetApp()->LoadString(uiText);
		XString sCaption;

		if (uiCaption)
		{
			sCaption = GetApp()->LoadString(uiCaption);
		}

		return Show(sText, uiCaption ? (LPCTSTR) sCaption : NULL, uiFlags, hWndParent);
	}

private:
// Internal functions...
	// Hook function centering the message box ("::MessageBox()" API)...
	inline static LRESULT CALLBACK _CBTProc(int iCode, WPARAM wParam, LPARAM lParam)
	{
		// Notification that a window is about to be activated...
		if (iCode != HCBT_ACTIVATE)
			return ::CallNextHookEx(s_hHook, iCode, wParam, lParam);

		// Get window handle of the message box...
		HWND hWnd  = reinterpret_cast<HWND>(wParam);
		if (!::IsWindow(hWnd))
			return ::CallNextHookEx(s_hHook, iCode, wParam, lParam);

		// Center window...
		XWindow wnd(hWnd);
		wnd.CenterWindow(::GetParent(hWnd));
		wnd.Detach();

		// Unhook...
		::UnhookWindowsHookEx(s_hHook);

		s_hHook = NULL;

		return 0;
	}

private:
// Internal data...
	static __declspec(thread) HHOOK s_hHook;
};

__declspec(selectany) HHOOK XMessageBox::s_hHook = NULL;



// #######################################################################################



// ***************************************************************************************
// XPropertyPageT...
// ***************************************************************************************
template
<
	typename t_Type,
	WORD t_IDD,
	typename t_Base = XWindowBaseT<CPropertyPageWindow>
>
class XPropertyPageT : public CPropertyPageImpl<t_Type, t_Base>
{
// Message map...
BEGIN_MSG_MAP(XPropertyPageT)
	MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
END_MSG_MAP()

public:
// Construction/Destruction...
	inline XPropertyPageT()
	{
	}

	inline ~XPropertyPageT()
	{
	}

public:
// Shared data...
	// Property page ID...
	enum _IDD
	{
		IDD = t_IDD
	};

protected:
// Message handler...
	inline LRESULT OnInitDialog(UINT uiMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;

		// Post init message to the property sheet to handle initialization...
		GetPropertySheet().SendMessage(WM_XInitPropertySheet);

		// Apply theme if appropriate...
		XTabCtrl wndTab(GetPropertySheet().GetTabControl());

		if (!(wndTab.GetStyle() & WS_DISABLED))
		{
			XTheme::EnableThemeDialogTexture(GetSafeHwnd());
		}

		return 0;
	}
};



// #######################################################################################



// ***************************************************************************************
// XPropertySheetT...
// ***************************************************************************************
template
<
	typename t_Type,
	typename t_Base = XWindowBaseT<CPropertySheetWindow>
>
class XPropertySheetT : public CPropertySheetImpl<t_Type, t_Base>
{
typedef CPropertySheetImpl<t_Type, t_Base> _Base;
// Message map...
BEGIN_MSG_MAP(XPropertySheetT)
	MESSAGE_HANDLER(WM_XInitPropertySheet, OnInitPropertySheet)
	CHAIN_MSG_MAP(_Base)
END_MSG_MAP()

public:
// Construction/Destruction...
	inline XPropertySheetT()
	{
		m_bInit = false;
	}

	inline ~XPropertySheetT()
	{
	}

protected:
// Message handler...
	inline LRESULT OnInitPropertySheet(UINT uiMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;

		if (m_bInit)
		{
			return 0;
		}

		if (!(GetStyle() & WS_CHILD))
		{
			CenterWindow();
		}

		m_bInit = true;

		if (!(m_psh.dwFlags & (PSH_WIZARD | PSH_WIZARD97)))
		{
			int iCount = GetPageCount();
			for (int i = 0; i < iCount; i++)
			{
				HWND hWndPage = IndexToHwnd(i);
				if (::IsWindow(hWndPage))
				{
					// Apply theme...
					XTheme::EnableThemeDialogTexture(hWndPage);
				}
			}
		}

		return 0;
	}

private:
// Internal data...
	xbool m_bInit;
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif X_WINDOWS



// #######################################################################################



// ***************************************************************************************
// XClipboard...
// ***************************************************************************************
class XClipboard
{
public:
	// Construction/Destruction...
	inline XClipboard()
	{
	}

	inline ~XClipboard()
	{
	}

public:
// Operations...
	// Copy "pszText" to clipboard...
	inline static xresult Copy(xcchart* pszText, HWND hWnd = NULL)
	{
		_XAssert(pszText);
		if (!pszText)
			return xeInvalidArgument;

		if (!::OpenClipboard(hWnd) ||
			!::EmptyClipboard())
			return _XResultLastError;

		xcuint32 uiLength = (xuint32) _xstrlent(pszText);

		HGLOBAL hGlobal = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, (uiLength + 1) * sizeof(xchart)); 
		if (!hGlobal)
		{
			::CloseClipboard();	

			return xeOutOfMemory;
		}

		xvoid* pBuffer = ::GlobalLock(hGlobal); 
		if (!pBuffer)
		{
			::GlobalFree(hGlobal);
			::CloseClipboard();

			return _XResultLastError;
		}

		_XMemoryCopy(pBuffer, pszText, uiLength * sizeof(xchart)); 

		if (!::GlobalUnlock(hGlobal))
		{
			xcdword dw = ::GetLastError();

			if (dw != NO_ERROR)
			{
				::GlobalFree(hGlobal);
				::CloseClipboard();

				return _XResultFromWin32(dw);
			}
		}

		// Place the handle on the clipboard...
#ifdef X_UNICODE
		::SetClipboardData(CF_UNICODETEXT, hGlobal); 
#else // UNICODE
		::SetClipboardData(CF_TEXT, hGlobal); 
#endif

		::CloseClipboard();

		return xsOK;
	}

	// Empty the clipboard...
	inline static xresult Empty(HWND hWnd = NULL)
	{
		if (!::OpenClipboard(hWnd) ||
			!::EmptyClipboard() ||
			!::CloseClipboard())
			return _XResultLastError;

		return xsOK;
	}
};



// #######################################################################################



#pragma comment(lib, "Version")

// ***************************************************************************************
// XFileVersionInfo: Helper object for file version info handling...
// ***************************************************************************************
class XFileVersionInfo  
{
public:
// Construction/Destruction...
	inline XFileVersionInfo()
	{
	}
	inline ~XFileVersionInfo()
	{
	}

public:
// Operations...
	inline xresult Init(xcchart* pszFileName)
	{
		_XAssert(pszFileName);
		if (!pszFileName)
			return xeInvalidArgument;

		// Get file version info data...
		XString sFileName(pszFileName);
		xvoid* pData = NULL;
		xresult xr = _GetFileVersionInfoBuffer(sFileName, &pData);
		if (_XFailed(xr))
			return xr;

		// Read the list of languages and code pages...
		XPtrT<_LangCodepage> pLangCodepage;
		xuint32 uiLength = 0;
		if (!::VerQueryValue(pData, __XT("\\VarFileInfo\\Translation"), (xvoid**) &pLangCodepage, &uiLength))
		{
			return _XResultLastError;
		}

		_GetFileVersionInfo(__XT("Comments"), uiLength, pData, pLangCodepage, m_sComments);
		_GetFileVersionInfo(__XT("CompanyName"), uiLength, pData, pLangCodepage, m_sCompanyName);
		_GetFileVersionInfo(__XT("FileDescription"), uiLength, pData, pLangCodepage, m_sFileDescription);
		_GetFileVersionInfo(__XT("FileVersion"), uiLength, pData, pLangCodepage, m_sFileVersion);
		_GetFileVersionInfo(__XT("InternalName"), uiLength, pData, pLangCodepage, m_sInternalName);
		_GetFileVersionInfo(__XT("LegalCopyright"), uiLength, pData, pLangCodepage, m_sLegalCopyright);
		_GetFileVersionInfo(__XT("LegalTrademarks"), uiLength, pData, pLangCodepage, m_sLegalTrademarks);
		_GetFileVersionInfo(__XT("OriginalFilename"), uiLength, pData, pLangCodepage, m_sOriginalFilename);
		_GetFileVersionInfo(__XT("PrivateBuild"), uiLength, pData, pLangCodepage, m_sPrivateBuild);
		_GetFileVersionInfo(__XT("ProductName"), uiLength, pData, pLangCodepage, m_sProductName);
		_GetFileVersionInfo(__XT("ProductVersion"), uiLength, pData, pLangCodepage, m_sProductVersion);
		_GetFileVersionInfo(__XT("SpecialBuild"), uiLength, pData, pLangCodepage, m_sSpecialBuild);

		SHFILEINFO sfi;
		_XMemoryZero(&sfi, sizeof(SHFILEINFO));
		if (::SHGetFileInfo(sFileName, 0, &sfi, sizeof(sfi), SHGFI_DISPLAYNAME | SHGFI_ICON))
		{
			m_iconBig = sfi.hIcon;
		}
		if (::SHGetFileInfo(sFileName, 0, &sfi, sizeof(sfi), SHGFI_DISPLAYNAME | SHGFI_ICON | SHGFI_SMALLICON))
		{
			m_iconSmall = sfi.hIcon;
		}

		return xsOK;
	}

	inline XString GetComments()
	{
		return m_sComments;
	}

	inline XString GetCompanyName()
	{
		return m_sCompanyName;
	}

	inline XString GetFileDescription()
	{
		return m_sFileDescription;
	}

	inline XString GetFileVersion()
	{
		return m_sFileVersion;
	}

	inline XString GetInternalName()
	{
		return m_sInternalName;
	}

	inline XString GetLegalCopyright()
	{
		return m_sLegalCopyright;
	}

	inline XString GetLegalTrademarks()
	{
		return m_sLegalTrademarks;
	}

	inline XString GetOriginalFilename()
	{
		return m_sOriginalFilename;
	}

	inline XString GetPrivateBuild()
	{
		return m_sPrivateBuild;
	}

	inline XString GetProductName()
	{
		return m_sProductName;
	}

	inline XString GetProductVersion()
	{
		return m_sProductVersion;
	}

	inline XString GetSpecialBuild()
	{
		return m_sSpecialBuild;
	}

	inline HICON GetIcon(xcbool bSmall = true)
	{
		if (!bSmall && m_iconBig)
		{
			return m_iconBig;
		}
		else if (bSmall && m_iconSmall)
		{
			return m_iconSmall;
		}
		else
			return NULL;
	}

private:
// Internal data...
	struct _LangCodepage
	{
		WORD wLanguage;
		WORD wCodepage;
	};

private:
// Internal functions...
	inline xresult _GetFileVersionInfoBuffer(xcchart* pszFileName, xvoid** pData)
	{
		_XAssert(pszFileName && pData);
		if (!pszFileName || !pData)
			return xeInvalidArgument;

		xdword dwHandle = 0;
		xcdword dwLength = ::GetFileVersionInfoSize(pszFileName, &dwHandle);
		if (dwLength)
		{	
			xvoid* pDataTemp = _xalloc(dwLength);
			if (!pDataTemp)
				return xeOutOfMemory;

			if (::GetFileVersionInfo(pszFileName, 0, dwLength, pDataTemp))
			{
				*pData = pDataTemp;

				return xsOK;
			}
			else
			{
				_xfree(pDataTemp);

				return _XResultLastError;
			}
		}

		return _XResultLastError;
	}
	
	inline xresult _GetFileVersionInfo(xcchart* pszInfo, xcuint32 uiLength, xvoid* pData, _LangCodepage* pLangCodepage, XString& sResult)
	{
		_XAssert(pszInfo && pData && pLangCodepage);
		if (!pszInfo || !pData || !pLangCodepage)
			return E_INVALIDARG;

		for (xuint32 ui = 0; ui < (uiLength / sizeof(_LangCodepage)); ui++)
		{
			// Format query...
			XString sQuery;
			sQuery.Format(_T("\\StringFileInfo\\%04x%04x\\%s"), pLangCodepage[ui].wLanguage, pLangCodepage[ui].wCodepage, pszInfo);

			// Retrieve file info for language and code page "i"...
			xuint32 uiBufferLength = 0;
			xchart* pszBuffer = NULL;
			if (::VerQueryValue(pData, (xchart*) ((xcchart*) sQuery), (xvoid**) &pszBuffer, &uiBufferLength) > 0)
			{
				// Success...
				sResult = pszBuffer;

				return xsOK;
			}
		}

		return xeFail;
	}

private:
// Internal data...
	XString m_sComments;
	XString m_sCompanyName;
	XString m_sFileDescription;
	XString m_sFileVersion;
	XString m_sInternalName;
	XString m_sLegalCopyright;
	XString m_sLegalTrademarks;
	XString m_sOriginalFilename;
	XString m_sPrivateBuild;
	XString m_sProductName;
	XString m_sProductVersion;
	XString m_sSpecialBuild;
	XIcon m_iconBig;
	XIcon m_iconSmall;
};



// #######################################################################################
